use std::sync::Arc;

use btleplug::api::{Central as _, Peripheral as _};
use state::{ApplicationState, CentralState, ChannelState, PeripheralItem};
use tauri::{async_runtime::RwLock, AppHandle, Emitter, State};

use crate::{bluetooth, errors, state::AppState};

mod state;

#[tauri::command]
pub async fn activate_central_adapter(
    app_handle: AppHandle,
    app_state: State<'_, Arc<RwLock<AppState>>>,
) -> Result<(), errors::AppError> {
    let app = Arc::new(app_handle);
    let adapter = bluetooth::get_central_adapter().await?;
    {
        let state = app_state.read().await;
        state.set_central_adapter(adapter).await;
        state.clear_central_event_handler().await;
    }
    let handle =
        bluetooth::handle_bluetooth_events(Arc::clone(&app), Arc::clone(&app_state)).await?;
    {
        let state = app_state.write().await; // Changed from lock() to write()
        state.set_central_event_handler(handle).await;
    }
    app.emit("app_state_updated", ()).unwrap();
    Ok(())
}

#[tauri::command]
pub async fn refresh_application_state(
    app_state: State<'_, Arc<RwLock<AppState>>>,
) -> Result<ApplicationState, errors::AppError> {
    let state = app_state.read().await;
    let mut peripherals: Vec<PeripheralItem> = vec![];
    let mut connected_peripheral = None;

    let central = state.get_central_adapter().await;
    let central_state = if let Some(central) = central {
        let central_device_state = central
            .adapter_state()
            .await
            .map_err(|_| errors::AppError::BluetoothNotReady)?;

        let found_peripherals = central
            .peripherals()
            .await
            .map_err(|_| errors::AppError::UnableToRetrievePeripherals)?;
        for peripheral in found_peripherals {
            let properties = peripheral
                .properties()
                .await
                .map_err(|_| errors::AppError::UnableToRetrievePeripheralProperties)?;
            if let Some(properties) = properties {
                let represent = properties
                    .local_name
                    .unwrap_or_else(|| properties.address.to_string());
                let item = PeripheralItem {
                    id: peripheral.id(),
                    address: properties.address.to_string(),
                    represent,
                    is_connected: peripheral
                        .is_connected()
                        .await
                        .map_err(|_| errors::AppError::UnableToRetrievePeripheralProperties)?,
                    rssi: properties.rssi,
                    battery: properties.tx_power_level,
                };
                if peripheral
                    .is_connected()
                    .await
                    .map_err(|_| errors::AppError::UnableToRetrievePeripheralState)?
                {
                    connected_peripheral = Some(item.clone());
                }
                peripherals.push(item);
            }
        }

        CentralState {
            is_ready: central_device_state == btleplug::api::CentralState::PoweredOn,
            is_scanning: *state.scanning.lock().await,
            connected: state
                .connected_peripheral
                .lock()
                .await
                .clone()
                .map(|p| p.id()),
        }
    } else {
        CentralState::default()
    };

    Ok(ApplicationState {
        central: central_state,
        peripherals,
        connected_peripheral,
        channel_a: ChannelState::default(),
        channel_b: ChannelState::default(),
    })
}

#[tauri::command]
pub async fn start_scan_devices(
    app_handle: AppHandle,
    app_state: State<'_, Arc<RwLock<AppState>>>,
) -> Result<(), errors::AppError> {
    bluetooth::start_scan(Arc::new(app_handle), app_state).await?;
    Ok(())
}

#[tauri::command]
pub async fn stop_scan_devices(
    app_handle: AppHandle,
    app_state: State<'_, Arc<RwLock<AppState>>>,
) -> Result<(), errors::AppError> {
    bluetooth::stop_scan(Arc::new(app_handle), app_state).await?;
    Ok(())
}
